@@ -94,21 +94,26 @@ static int Elf_(r_bin_elf_init_shdr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 }
 
 static int Elf_(r_bin_elf_init_strtab)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	int sz;
 	if (!bin->shdr)
 		return R_FALSE;
 	bin->shstrtab_section =
 	bin->strtab_section = &bin->shdr[bin->ehdr.e_shstrndx];
 	bin->shstrtab_size =
 	bin->strtab_size = bin->strtab_section->sh_size;
-	if ((bin->strtab = (char *)malloc (bin->strtab_section->sh_size)) == NULL) {
+	sz = sizeof (struct r_bin_elf_section_t) + bin->strtab_section->sh_size;
+	if ((bin->strtab = (char *)malloc (sz)) == NULL) {
 		perror ("malloc");
+		bin->shstrtab = NULL;
 		return R_FALSE;
 	}
+	memset (bin->strtab, 0, sz);
 	bin->shstrtab = bin->strtab;
 	if (r_buf_read_at (bin->b, bin->strtab_section->sh_offset, (ut8*)bin->strtab,
 				bin->strtab_section->sh_size) == -1) {
 		eprintf ("Error: read (strtab)\n");
 		R_FREE (bin->strtab);
+		bin->shstrtab = NULL;
 		return R_FALSE;
 	}
 	return R_TRUE;
@@ -208,9 +213,12 @@ static ut64 Elf_(r_bin_elf_get_section_addr)(struct Elf_(r_bin_elf_obj_t) *bin,
 	int i;
 	if (!bin->shdr || !bin->strtab)
 		return -1;
-	for (i = 0; i < bin->ehdr.e_shnum; i++)
+	for (i = 0; i < bin->ehdr.e_shnum; i++) {
+		if (bin->shdr[i].sh_name > bin->shstrtab_section->sh_size)
+			continue;
 		if (!strcmp (&bin->strtab[bin->shdr[i].sh_name], section_name))
 			return (ut64)bin->shdr[i].sh_addr;
+	}
 	return -1;
 }
 
@@ -277,6 +285,36 @@ ut64 Elf_(r_bin_elf_get_baddr)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	return 0;
 }
 
+ut64 Elf_(r_bin_elf_get_init_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	ut64 entry = Elf_(r_bin_elf_get_entry_offset) (bin);
+	ut8 buf[512];
+
+	if (r_buf_read_at (bin->b, entry+16, buf, sizeof (buf)) == -1) {
+		eprintf ("Error: read (entry)\n");
+		return 0;
+	}
+	if (buf[0] == 0x68) { // push // x86 only
+		memmove (buf, buf+1, 4);
+		return (ut64)((int)(buf[0]+(buf[1]<<8)+(buf[2]<<16)+(buf[3]<<24)))-bin->baddr;
+	}
+	return 0;
+}
+
+ut64 Elf_(r_bin_elf_get_fini_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	ut64 entry = Elf_(r_bin_elf_get_entry_offset) (bin);
+	ut8 buf[512];
+
+	if (r_buf_read_at (bin->b, entry+11, buf, sizeof (buf)) == -1) {
+		eprintf ("Error: read (entry)\n");
+		return 0;
+	}
+	if (buf[0] == 0x68) { // push // x86/32 only
+		memmove (buf, buf+1, 4);
+		return (ut64)((int)(buf[0]+(buf[1]<<8)+(buf[2]<<16)+(buf[3]<<24)))-bin->baddr;
+	}
+	return 0;
+}
+
 ut64 Elf_(r_bin_elf_get_entry_offset)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	if (bin->ehdr.e_entry < bin->baddr)
 		return bin->ehdr.e_entry;
@@ -353,6 +391,7 @@ char* Elf_(r_bin_elf_get_data_encoding)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	}
 }
 
+// TODO: do not strdup here
 char* Elf_(r_bin_elf_get_arch)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	switch (bin->ehdr.e_machine) {
 	case EM_AVR:
@@ -494,9 +533,26 @@ int Elf_(r_bin_elf_get_bits)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	}
 }
 
+static inline int needle(struct Elf_(r_bin_elf_obj_t) *bin, const char *s) {
+	if (bin->shstrtab)
+		return r_mem_mem ((const ut8*)bin->shstrtab, bin->shstrtab_size,
+				(const ut8*)s, strlen (s)) != NULL;
+	return 0;
+}
+
 // TODO: must return const char * all those strings must be const char os[LINUX] or so
 char* Elf_(r_bin_elf_get_osabi_name)(struct Elf_(r_bin_elf_obj_t) *bin) {
+	/* Hack to identify OS */
+	if (needle (bin, "openbsd")) return strdup ("openbsd");
+	if (needle (bin, "netbsd")) return strdup ("netbsd");
+	if (needle (bin, "freebsd")) return strdup ("freebsd");
+	if (needle (bin, "GNU")) return strdup ("linux");
+	return strdup ("linux");
+#if 0
+	// XXX: this is wrong. openbsd bins are identified as linux ones.
 	switch (bin->ehdr.e_ident[EI_OSABI]) {
+	case ELFOSABI_ARM_AEABI:
+	case ELFOSABI_ARM:        return strdup ("arm");
 	case ELFOSABI_NONE:       return strdup ("linux"); // sysv
 	case ELFOSABI_HPUX:       return strdup ("hpux");
 	case ELFOSABI_NETBSD:     return strdup ("netbsd");
@@ -509,9 +565,9 @@ char* Elf_(r_bin_elf_get_osabi_name)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	case ELFOSABI_MODESTO:    return strdup ("modesto");
 	case ELFOSABI_OPENBSD:    return strdup ("openbsd");
 	case ELFOSABI_STANDALONE: return strdup ("standalone");
-	case ELFOSABI_ARM:        return strdup ("arm");
 	default:                  return r_str_dup_printf ("<unknown: %x>", bin->ehdr.e_ident[EI_OSABI]);
 	}
+#endif
 }
 
 int Elf_(r_bin_elf_is_big_endian)(struct Elf_(r_bin_elf_obj_t) *bin) {
@@ -739,7 +795,7 @@ struct r_bin_elf_lib_t* Elf_(r_bin_elf_get_libs)(struct Elf_(r_bin_elf_obj_t) *b
 
 struct r_bin_elf_section_t* Elf_(r_bin_elf_get_sections)(struct Elf_(r_bin_elf_obj_t) *bin) {
 	struct r_bin_elf_section_t *ret = NULL;
-	int i;
+	int i, nidx;
 	
 	if (!bin->shdr)
 		return NULL;
@@ -752,8 +808,13 @@ struct r_bin_elf_section_t* Elf_(r_bin_elf_get_sections)(struct Elf_(r_bin_elf_o
 		ret[i].size = bin->shdr[i].sh_size;
 		ret[i].align = bin->shdr[i].sh_addralign;
 		ret[i].flags = bin->shdr[i].sh_flags;
-		strncpy (ret[i].name, bin->shstrtab?
-			&bin->shstrtab[bin->shdr[i].sh_name]: "unknown", ELF_STRING_LENGTH);
+		//memset (ret[i].name, 0, sizeof (ret[i].name));
+		nidx = bin->shdr[i].sh_name;
+		if (nidx<0 || nidx>bin->shstrtab_section->sh_size)
+			strncpy (ret[i].name, "invalid", sizeof (ret[i].name)-4);
+		else strncpy (ret[i].name, bin->shstrtab?
+			&bin->shstrtab[bin->shdr[i].sh_name]: "unknown", sizeof (ret[i].name)-4);
+		ret[i].name[sizeof (ret[i].name)-2] = 0;
 		ret[i].last = 0;
 	}
 	ret[i].last = 1;
