@@ -4,7 +4,7 @@
 
 #define NPF 6
 static int blocksize = 0;
-static const char *printfmt[] = { "x", "pd", "f tmp;sr sp;x 64;dr=;s-;s tmp;f-tmp;pd", "p8", "pc", "ps" };
+static const char *printfmt[] = { "x", "pd", "f tmp;sr sp;pw 64;dr=;s-;s tmp;f-tmp;pd", "p8", "pc", "ps" };
 static int autoblocksize = 1;
 static int obs = 0;
 
@@ -49,7 +49,6 @@ R_API void r_core_visual_prompt (RCore *core) {
 	if (curset) r_core_seek (core, oseek, 1);
 }
 
-
 static int visual_fkey(RCore *core, int ch) {
 	const char *cmd;
 	switch (ch) {
@@ -169,6 +168,7 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 		r_cons_flush ();
 		r_cons_set_raw (0);
 		strcpy (buf, "wx ");
+		r_line_set_prompt ("hex: ");
 		if (r_cons_fgets (buf+3, 1000, 0, NULL) <0) buf[0]='\0';
 		if (*buf) {
 			if (curset) r_core_seek (core, core->offset + cursor, 0);
@@ -200,13 +200,24 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 		r_io_sundo_push (core->io);
 		break;
 	case 'G':
+{
+		int ret = 0;
 		if (core->io->va) {
-			ut64 offset = r_io_section_get_vaddr (core->io, core->file->size-core->blocksize);
-			r_core_seek (core, offset, 1);
-		} else
-			r_core_seek (core, core->file->size-core->blocksize, 1);
-		r_io_sundo_push (core->io);
-		//r_core_cmd(core, "s 0", 0);
+			ut64 offset = r_io_section_get_vaddr (core->io, 0);
+			//ut64 offset = r_io_section_get_vaddr (core->io,
+			//	core->file->size-core->blocksize);
+			if (offset == UT64_MAX) {
+				ret = r_core_seek (core, offset, 1);
+				memset (core->block, 0xff, core->blocksize);
+			} else {
+				offset += core->file->size - core->blocksize;
+				ret = r_core_seek (core, offset, 1);
+				memset (core->block, 0xff, core->blocksize);
+			}
+		} else ret = r_core_seek (core, core->file->size-core->blocksize, 1);
+		if (ret != -1)
+			r_io_sundo_push (core->io);
+}
 		break;
 	case 'h':
 		if (curset) {
@@ -369,14 +380,15 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 		core->printidx = (core->printidx-1)%NPF;
 		break;
 	case 'm':
-		r_core_visual_mark (core, r_cons_readchar());
+		r_core_visual_mark (core, r_cons_readchar ());
 		break;
 	case '\'':
-		r_core_visual_mark_seek (core, r_cons_readchar());
+		r_core_visual_mark_seek (core, r_cons_readchar ());
 		break;
 	case 'y':
 		if (ocursor==-1) r_core_yank (core, core->offset+cursor, 1);
-		else r_core_yank (core, core->offset+((ocursor<cursor)?ocursor:cursor), R_ABS (cursor-ocursor)+1);
+		else r_core_yank (core, core->offset+((ocursor<cursor)?
+			ocursor:cursor), R_ABS (cursor-ocursor)+1);
 		break;
 	case 'Y':
 		if (!core->yank) {
@@ -391,7 +403,8 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 			if (cur>=core->blocksize)
 				cur = core->print->cur-1;
 			if (ocursor==-1) sprintf (buf, "wos 01 @ $$+%i:1",cursor);
-			else sprintf (buf, "wos 01 @ $$+%i:%i", cursor<ocursor?cursor:ocursor, R_ABS (ocursor-cursor)+1);
+			else sprintf (buf, "wos 01 @ $$+%i:%i", cursor<ocursor?
+				cursor:ocursor, R_ABS (ocursor-cursor)+1);
 			r_core_cmd (core, buf, 0);
 		} else r_core_block_size (core, core->blocksize-1);
 		break;
@@ -401,7 +414,8 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 			if (cur>=core->blocksize)
 				cur = core->print->cur-1;
 			if (ocursor==-1) sprintf (buf, "woa 01 @ $$+%i:1", cursor);
-			else sprintf (buf, "woa 01 @ $$+%i:%i", cursor<ocursor?cursor:ocursor, R_ABS (ocursor-cursor)+1);
+			else sprintf (buf, "woa 01 @ $$+%i:%i",
+				cursor<ocursor? cursor: ocursor, R_ABS (ocursor-cursor)+1);
 			r_core_cmd (core, buf, 0);
 		} else r_core_block_size (core, core->blocksize+1);
 		break;
@@ -436,7 +450,7 @@ R_API int r_core_visual_cmd(RCore *core, int ch) {
 		r_cons_printf ("Enter a comment: ('-' to remove, '!' to use $EDITOR)\n");
 		r_cons_flush ();
 		r_cons_set_raw (R_FALSE);
-		strcpy (buf, "CC 0 ");
+		strcpy (buf, "CC ");
 		r_line_set_prompt ("comment: ");
 		i = strlen (buf);
 		if (r_cons_fgets (buf+i, sizeof (buf)-i-1, 0, NULL) >1) {
@@ -536,7 +550,7 @@ R_API void r_core_visual_title (RCore *core, int color) {
 		break;
 	case 1: // pd
 	case 2: // pd+dbg
-		r_core_block_size (core, core->cons->rows * 15);
+		r_core_block_size (core, core->cons->rows * 5); // this is hacky
 		break;
 	}
 
@@ -558,8 +572,8 @@ R_API void r_core_visual_title (RCore *core, int color) {
 static void r_core_visual_refresh (RCore *core) {
 	const char *vi;
 	r_cons_get_size (NULL);
-	r_cons_clear00 ();
 	r_print_set_cursor (core->print, curset, ocursor, cursor);
+	r_cons_clear00 ();
 
 	vi = r_config_get (core->config, "cmd.vprompt");
 	if (vi) r_core_cmd (core, vi, 0);
